##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

"""
    Type registration tool interface description.
"""
__version__='$Revision: 1.12 $'[11:-2]

import OFS
from Globals import default__class_init__, HTMLFile, package_home
from utils import UniqueObject, SimpleItemWithProperties
import os, string
from AccessControl import getSecurityManager
from Acquisition import aq_base

_dtmldir = os.path.join( package_home( globals() ), 'dtml' )


class ContentFactoryMetadata (SimpleItemWithProperties):
    """
    Portal content factory.
    """
    meta_type = 'Content Factory Metadata'
    _isTypeInformation = 1

    __ac_permissions__= (('View', ('Type',
                                   'Description',
                                   'allowed',
                                   'getId',
                                   )),
                         ('Manage portal', ('manage_editProperties',
                                            'manage_changeProperties',
                                            'manage_propertiesForm',
                                            )),
                        )

    _properties = ({'id':'title', 'type': 'string', 'mode':'w',
                    'label':'Title'},
                   {'id':'description', 'type': 'text', 'mode':'w',
                    'label':'Description'},
                   {'id':'content_meta_type', 'type': 'string', 'mode':'w',
                    'label':'Meta type'},
                   {'id':'product', 'type': 'string', 'mode':'w',
                    'label':'Product name'},
                   {'id':'factory', 'type': 'string', 'mode':'w',
                    'label':'Factory method in product'},
                   {'id':'immediate_view', 'type': 'string', 'mode':'w',
                    'label':'Initial view name'},
                   )

    title = ''
    description = ''
    content_meta_type = ''
    product = ''
    factory = ''
    immediate_view = ''

    def __init__( self, id, title='', description='', meta_type='',
                  product='', factory='', immediate_view=''):
        self.id = id
        if title: self.title = title
        if description: self.description = description
        if meta_type: self.content_meta_type = meta_type
        else: self.content_meta_type = self.id
        if product: self.product = product
        if factory: self.factory = factory
        if immediate_view: self.immediate_view = immediate_view

    #
    #   Accessors
    #
    def Type( self ):
        """
            Return the "human readable" type name (note that it
            may not map exactly to the 'meta_type', e.g., for
            l10n/i18n or where a single content class is being
            used twice, under different names.
        """
        if self.title:
            return self.title
        else:
            return self.id
    
    def Description( self ):
        """
            Textual description of the class of objects (intended
            for display in a "constructor list").
        """
        return self.description

    def Metatype (self):
        """
            Returns the Zope 'meta_type' for this content object.
            May be used for building the list of portal content
            meta types.
        """
        return self.content_meta_type
    
    def getId(self):
        return self.id

    def _getFactoryMethod(self, container, raise_exc=0):
        if not self.product or not self.factory:
            return None
        p = container.manage_addProduct[self.product]
        m = getattr(p, self.factory)
        try:
            if getSecurityManager().validate(p, p, self.factory, m):
                return m
        except:
            if raise_exc:
                raise
        return None

    def allowed( self, container, raise_exc=0):
        """
        Does the current user have the permission required in
        order to construct an instance?
        """
        m = self._getFactoryMethod(container, raise_exc)
        return (m is not None)
        
    #
    #   Agent methods
    #
    constructInstance__roles__ = None  # Public.
    def constructInstance( self, container, id ):
        """
        Build a "bare" instance of the appropriate type in
        'container', using 'id' as its id.  Return the URL
        of its "immediate" view (typically the metadata form).
        """
        # Get the factory method, performing a security check
        # in the process.
        m = self._getFactoryMethod(container, raise_exc=1)
        id = str(id)
        m(id=id)
        ob = container._getOb(id)
        return '%s/%s' % ( ob.absolute_url(), self.immediate_view )

default__class_init__( ContentFactoryMetadata )


class ContentTypeInformation( OFS.SimpleItem.SimpleItem ):
    """
        Registry entry.
    """
    meta_type = 'ContentTypeInformation'
    _isTypeInformation = 1

    __ac_permissions__= ( ( 'View'
                          , ( 'Metatype'
                            , 'Type'
                            , 'Description'
                            , 'allowed'
                            , 'getId'
                            )
                          , ( 'Anonymous','Manager')
                          )
                        , ( 'Manage portal'
                          , ( 'editForm'
                            , 'edit'
                            )
                          )
                        )

    manage_options = ( ( { 'label'  : 'Properties'
                         , 'action' : 'manage_editForm'
                         }
                       ,
                       )
                     + OFS.SimpleItem.SimpleItem.manage_options
                     )

    def __init__( self
                , id
                , content_meta_type
                , display_type
                , description
                , permission
                , constructor_path
                , immediate_view
                ):
        self.id = id
        self.edit( content_meta_type
                 , display_type
                 , description
                 , permission
                 , constructor_path
                 , immediate_view
                 )

    #
    #   Accessors
    #
    def Metatype( self ):
        """
            Return the Zope 'meta_type' for this content object.
        """
        return self.content_meta_type
    
    def Type( self ):
        """
            Return the "human readable" type name (note that it
            may not map exactly to the 'meta_type', e.g., for
            l10n/i18n or where a single content class is being
            used twice, under different names.
        """
        return self.display_type
    
    def Description( self ):
        """
            Textual description of the class of objects (intended
            for display in a "constructor list").
        """
        return self.description
    
    def allowed( self, container ):
        """
        Does the current user have the permission required in
        order to construct an instance?
        """
        permission = self.permission
        if permission and not getSecurityManager().checkPermission(
            permission, container):
            return 0
        return 1

    def getId(self):
        return self.id

    #
    #   Mutators
    #
    manage_editForm = HTMLFile( 'typeinfoEdit', _dtmldir )
    def edit( self
            , content_meta_type
            , display_type
            , description
            , permission
            , constructor_path
            , immediate_view
            , REQUEST=None
            ):
        """
            Update the info object.
        """
        self.content_meta_type  = str( content_meta_type )
        self.display_type       = str( display_type )
        self.description        = str( description )
        self.permission         = str( permission )
        self.constructor_path   = string.split( constructor_path, '/' )
        self.immediate_view     = str( immediate_view )

        if REQUEST is not None:
            REQUEST[ 'RESPONSE' ].redirect( self.absolute_url()
                                          + '/manage_editForm' )

    #
    #   Agent methods
    #
    constructInstance__roles__ = None  # Public.
    def constructInstance( self, container, id ):
        """
        Build a "bare" instance of the appropriate type in
        'container', using 'id' as its id.  Return the URL
        of its "immediate" view (typically the metadata form).
        """
        if not self.allowed(container):
            raise 'Unauthorized'

        constructor = self.restrictedTraverse( self.constructor_path )
        #   Rewrap to get into container's context.
        constructor = constructor.aq_base.__of__( container )
        id = str(id)
        object = constructor( container, id )
        return '%s/%s' % ( object.absolute_url(), self.immediate_view )

default__class_init__( ContentTypeInformation )



allowedTypes = ( 'Script (Python)'
                 , 'Python Method'
                 , 'DTML Method'
                 , 'External Method'
                 , ContentFactoryMetadata.meta_type
                 , ContentTypeInformation.meta_type
                 )

class TypesTool( UniqueObject, OFS.Folder.Folder ):
    """
        Provides a configurable registry of portal content types.
    """
    id = 'portal_types'
    meta_type = 'PTK Basic Types Tool'

    __ac_permissions__= (('Access contents information', ('getTypeInfo',
                                                          'listTypeInfo')),
                         ('Manage portal', ('addCFM',
                                            'manage_addCFM',
                                            'addContentTypeInformationForm',
                                            'addContentTypeInformation',)),
                         )

    def all_meta_types(self):
        all = TypesTool.inheritedAttribute('all_meta_types')(self)
        cfm = ContentFactoryMetadata
        cti = ContentTypeInformation
        return (
            {'name':cfm.meta_type,
             'action':'addCFM',
             'permission':'Manage portal'},
            {'name':cti.meta_type,
             'action':'addContentTypeInformationForm',
             'permission':'Manage portal'},
            ) + all

    def filtered_meta_types(self, user=None):
        # Filters the list of available meta types.
        all = TypesTool.inheritedAttribute('filtered_meta_types')(self)
        meta_types = []
        for meta_type in self.all_meta_types():
            if meta_type['name'] in allowedTypes:
                meta_types.append(meta_type)
        return meta_types

    addCFM = HTMLFile( 'addCFM', _dtmldir )

    def manage_addCFM(self, id, RESPONSE=None):
        ' '
        id = str(id)
        ob = ContentFactoryMetadata(id)
        self._setObject(id, ob)
        if RESPONSE is not None:
            RESPONSE.redirect('%s/%s/manage_propertiesForm' % (
                self.absolute_url(), id))

    addContentTypeInformationForm = HTMLFile( 'typeinfoAdd', _dtmldir )
    addContentTypeInformationForm.__name__ = 'addContentTypeInformationForm'

    def addContentTypeInformation( self
                                   , id
                                   , content_meta_type
                                   , display_type
                                   , description
                                   , permission
                                   , constructor_path
                                   , immediate_view
                                   , RESPONSE=None
                                   ):
        """
        Create a CTI in 'self'.
        """
        cti = ContentTypeInformation( id
                                      , content_meta_type
                                      , display_type
                                      , description
                                      , permission
                                      , constructor_path
                                      , immediate_view
                                      )
        self._setObject( id, cti )

        if RESPONSE is not None:
            RESPONSE.redirect('%s/manage_main' % self.absolute_url())

    def getTypeInfo( self, contentType ):
        """
            Return an instance which implements the
            ContentTypeInformation interface, corresponding to
            the specified 'contentType'.
        """
        ob = getattr( self, contentType, None )
        if getattr(aq_base(ob), '_isTypeInformation', 0):
            return ob
        else:
            return None

    def listTypeInfo( self, container=None ):
        """
            Return a sequence of instances which implement the
            ContentTypeInformation interface, one for each content
            type regisetered in the portal.
        """
        rval = []
        sm = getSecurityManager()
        for t in self.objectValues():
            # Filter out things that aren't ContentTypeInformation and
            # types for which the user does not have adequate permission.
            if not getattr(aq_base(t), '_isTypeInformation', 0):
                continue
            if not t.Title():
                # Not ready.
                continue
            if container is not None:
                if not t.allowed(container):
                    continue
            rval.append(t)
        return rval
    
    constructContent__roles__ = None  # Public, enforces own security.
    def constructContent( self, type_name, container, id, RESPONSE=None ):
        """
            Build an instance of the appropriate content class in
            'container', using 'id'.
        """
        info = self.getTypeInfo( type_name )
        if info is None:
            raise 'ValueError', 'No such content type: %s' % contentType
        
        immediate_url = info.constructInstance( container, id )

        if RESPONSE is not None:
            RESPONSE.redirect( immediate_url )

default__class_init__( TypesTool )
